@@ -1,7 +1,7 @@
 // -*- mode: C++; c-file-style: "cc-mode" -*-
 //*************************************************************************
 //
-// Copyright 2003-2019 by Wilson Snyder. This program is free software; you can
+// Copyright 2003-2020 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License.
 // Version 2.0.
@@ -22,7 +22,7 @@
 ///     config_build.h.in, code needed by Verilated code only goes into
 ///     verilated.h, and code needed by both goes here (verilatedos.h).
 ///
-/// Code available from: http://www.veripool.org/verilator
+/// Code available from: https://verilator.org
 ///
 //*************************************************************************
 
@@ -122,7 +122,9 @@
 #endif
 
 #ifdef VL_THREADED
-# ifdef __GNUC__
+# if defined(_MSC_VER) && _MSC_VER >= 1900
+#  define VL_THREAD_LOCAL thread_local
+# elif defined(__GNUC__)
 #  if (__cplusplus < 201103L) && !defined(VL_THREADED_NO_C11_WARNING)
 #    error "VL_THREADED/--threads support requires C++-11 or newer only; use newer compiler"
 #  endif
@@ -152,15 +154,23 @@
 #define VL_UL(c) (static_cast<IData>(c##UL))  ///< Add appropriate suffix to 32-bit constant
 
 #if defined(VL_CPPCHECK) || defined(__clang_analyzer__)
-# define VL_DANGLING(v)
+# define VL_DANGLING(var)
 #else
-# define VL_DANGLING(v) do { (v) = NULL; } while(0)  ///< After e.g. delete, set variable to NULL to indicate must not use later
+///< After e.g. delete, set variable to NULL to indicate must not use later
+# define VL_DANGLING(var) do { (var) = NULL; } while(0)
 #endif
 
+///< Perform an e.g. delete, then set variable to NULL to indicate must not use later.
+///< Unlike VL_DO_CLEAR the setting of the variable is only for debug reasons.
+#define VL_DO_DANGLING(stmt, var) do { do { stmt; } while(0); VL_DANGLING(var); } while(0)
+
+///< Perform an e.g. delete, then set variable to NULL as a requirement
+#define VL_DO_CLEAR(stmt, stmt2) do { do { stmt; } while(0); do { stmt2; } while(0); } while(0)
+
 //=========================================================================
 // C++-2011
 
-#if __cplusplus >= 201103L || defined(__GXX_EXPERIMENTAL_CXX0X__)
+#if __cplusplus >= 201103L || defined(__GXX_EXPERIMENTAL_CXX0X__) || defined(VL_CPPCHECK)
 # define VL_EQ_DELETE = delete
 # define vl_unique_ptr std::unique_ptr
 // By default we use std:: types in C++11.
@@ -174,11 +184,15 @@
 #  define VL_INCLUDE_UNORDERED_MAP <unordered_map>
 #  define VL_INCLUDE_UNORDERED_SET <unordered_set>
 # endif
+# define VL_FINAL final
+# define VL_OVERRIDE override
 #else
 # define VL_EQ_DELETE
 # define vl_unique_ptr std::auto_ptr
 # define VL_INCLUDE_UNORDERED_MAP "verilated_unordered_set_map.h"
 # define VL_INCLUDE_UNORDERED_SET "verilated_unordered_set_map.h"
+# define VL_FINAL
+# define VL_OVERRIDE
 #endif
 
 //=========================================================================
@@ -209,9 +223,12 @@
 
 #ifdef __MINGW32__
 # define __USE_MINGW_ANSI_STDIO 1  // Force old MinGW (GCC 5 and older) to use C99 formats
-# define __STDC_FORMAT_MACROS   1  // Otherwise MinGW doesn't get PRId64 for fstapi.c
 #endif
 
+// The inttypes supplied with some GCC & MINGW32 versions requires STDC_FORMAT_MACROS
+// to be declared in order to get the PRIxx macros used by fstapi.c
+#define __STDC_FORMAT_MACROS
+
 #if defined(__CYGWIN__)
 
 # include <stdint.h>
@@ -315,16 +332,21 @@ typedef unsigned long long      vluint64_t;     ///< 64-bit unsigned type
 //=========================================================================
 // Integer size macros
 
-#define VL_BYTESIZE 8                   ///< Bits in a byte
-#define VL_SHORTSIZE 16                 ///< Bits in a short
-#define VL_WORDSIZE 32                  ///< Bits in a word
-#define VL_QUADSIZE 64                  ///< Bits in a quadword
-#define VL_WORDSIZE_LOG2 5              ///< log2(VL_WORDSIZE)
+#define VL_BYTESIZE 8                   ///< Bits in a CData / byte
+#define VL_SHORTSIZE 16                 ///< Bits in a SData / short
+#define VL_IDATASIZE 32                 ///< Bits in a IData / word
+#define VL_WORDSIZE IDATASIZE           ///< Legacy define
+#define VL_QUADSIZE 64                  ///< Bits in a QData / quadword
+#define VL_EDATASIZE 32                 ///< Bits in a EData (WData entry)
+#define VL_EDATASIZE_LOG2 5             ///< log2(VL_EDATASIZE)
+#define VL_CACHE_LINE_BYTES 64          ///< Bytes in a cache line (for alignment)
 
 /// Bytes this number of bits needs (1 bit=1 byte)
-#define VL_BYTES_I(nbits) (((nbits)+(VL_BYTESIZE-1))/VL_BYTESIZE)
-/// Words this number of bits needs (1 bit=1 word)
-#define VL_WORDS_I(nbits) (((nbits)+(VL_WORDSIZE-1))/VL_WORDSIZE)
+#define VL_BYTES_I(nbits) (((nbits) + (VL_BYTESIZE - 1)) / VL_BYTESIZE)
+/// Words/EDatas this number of bits needs (1 bit=1 word)
+#define VL_WORDS_I(nbits) (((nbits) + (VL_EDATASIZE - 1)) / VL_EDATASIZE)
+/// Words/EDatas a quad requires
+#define VL_WQ_WORDS_E VL_WORDS_I(VL_QUADSIZE)
 
 //=========================================================================
 // Class definition helpers
@@ -343,8 +365,9 @@ typedef unsigned long long      vluint64_t;     ///< 64-bit unsigned type
 //=========================================================================
 // Base macros
 
-#define VL_SIZEBITS_I (VL_WORDSIZE-1)   ///< Bit mask for bits in a word
-#define VL_SIZEBITS_Q (VL_QUADSIZE-1)   ///< Bit mask for bits in a quad
+#define VL_SIZEBITS_I (VL_IDATASIZE - 1)  ///< Bit mask for bits in a word
+#define VL_SIZEBITS_Q (VL_QUADSIZE - 1)  ///< Bit mask for bits in a quad
+#define VL_SIZEBITS_E (VL_EDATASIZE - 1)  ///< Bit mask for bits in a quad
 
 /// Mask for words with 1's where relevant bits are (0=all bits)
 #define VL_MASK_I(nbits)  (((nbits) & VL_SIZEBITS_I) \
@@ -352,9 +375,15 @@ typedef unsigned long long      vluint64_t;     ///< 64-bit unsigned type
 /// Mask for quads with 1's where relevant bits are (0=all bits)
 #define VL_MASK_Q(nbits)  (((nbits) & VL_SIZEBITS_Q) \
                          ? ((VL_ULL(1) << ((nbits) & VL_SIZEBITS_Q) )-VL_ULL(1)) : VL_ULL(~0))
-#define VL_BITWORD_I(bit) ((bit)/VL_WORDSIZE)  ///< Word number for a wide quantity
-#define VL_BITBIT_I(bit) ((bit)&VL_SIZEBITS_I)  ///< Bit number for a bit in a long
-#define VL_BITBIT_Q(bit) ((bit)&VL_SIZEBITS_Q)  ///< Bit number for a bit in a quad
+/// Mask for EData with 1's where relevant bits are (0=all bits)
+#define VL_MASK_E(nbits) VL_MASK_I(nbits)
+#define VL_EUL(n) VL_UL(n)  ///< Make constant number EData sized
+
+#define VL_BITWORD_I(bit) ((bit) / VL_IDATASIZE)  ///< Word number for sv DPI vectors
+#define VL_BITWORD_E(bit) ((bit) >> VL_EDATASIZE_LOG2)  ///< Word number for a wide quantity
+#define VL_BITBIT_I(bit) ((bit) & VL_SIZEBITS_I)  ///< Bit number for a bit in a long
+#define VL_BITBIT_Q(bit) ((bit) & VL_SIZEBITS_Q)  ///< Bit number for a bit in a quad
+#define VL_BITBIT_E(bit) ((bit) & VL_SIZEBITS_E)  ///< Bit number for a bit in a EData
 
 //=========================================================================
 // Floating point
@@ -390,7 +419,12 @@ typedef unsigned long long      vluint64_t;     ///< 64-bit unsigned type
 // Threading related OS-specific functions
 
 #if VL_THREADED
-# if defined(__i386__) || defined(__x86_64__)
+# ifdef _WIN32
+#  define WIN32_LEAN_AND_MEAN
+#  define NOMINMAX
+#  include "Windows.h"
+#  define VL_CPU_RELAX() YieldProcessor()
+# elif defined(__i386__) || defined(__x86_64__)
 /// For more efficient busy waiting on SMT CPUs, let the processor know
 /// we're just waiting so it can let another thread run
 #  define VL_CPU_RELAX() asm volatile("rep; nop" ::: "memory")
@@ -405,6 +439,21 @@ typedef unsigned long long      vluint64_t;     ///< 64-bit unsigned type
 # endif
 #endif
 
+//=========================================================================
+// String related OS-specific functions
+
+#ifdef _MSC_VER
+# define VL_STRCASECMP _stricmp
+#else
+# define VL_STRCASECMP strcasecmp
+#endif
+
+//=========================================================================
+// Stringify macros
+
+#define VL_STRINGIFY(x) VL_STRINGIFY2(x)
+#define VL_STRINGIFY2(x) #x
+
 //=========================================================================
 
 #endif  // Guard
